@@ -62,7 +62,9 @@ async def server_reflexive_candidate(protocol, stun_server):
         priority=candidate_priority(local_candidate.component, 'srflx'),
         host=response.attributes['XOR-MAPPED-ADDRESS'][0],
         port=response.attributes['XOR-MAPPED-ADDRESS'][1],
-        type='srflx')
+        type='srflx',
+        related_address=local_candidate.host,
+        related_port=local_candidate.port)
 
 
 def sort_candidate_pairs(pairs, ice_controlling):
@@ -126,14 +128,13 @@ class StunProtocol(asyncio.DatagramProtocol):
     def __init__(self, receiver):
         self.__closed = asyncio.Future()
         self.id = next_protocol_id()
-        self.queue = asyncio.Queue()
         self.receiver = receiver
         self.transport = None
         self.transactions = {}
 
     def connection_lost(self, exc):
         self.__log_debug('connection_lost(%s)', exc)
-        asyncio.ensure_future(self.queue.put(None))
+        self.receiver.data_received(None, self.local_candidate.component)
         self.__closed.set_result(True)
 
     def connection_made(self, transport):
@@ -149,8 +150,7 @@ class StunProtocol(asyncio.DatagramProtocol):
             self.__log_debug('< %s %s', addr, message)
         except ValueError:
             self.__log_debug('< %s DATA %d', addr, len(data))
-            coro = self.queue.put(data)
-            asyncio.ensure_future(coro)
+            self.receiver.data_received(data, self.local_candidate.component)
             return
 
         if ((message.message_class == stun.Class.RESPONSE or
@@ -170,13 +170,6 @@ class StunProtocol(asyncio.DatagramProtocol):
         self.transport.close()
         await self.__closed
 
-    async def recv_data(self):
-        return await self.queue.get(), self.local_candidate.component
-
-    async def send_data(self, data, addr):
-        self.__log_debug('> %s DATA %d', addr, len(data))
-        self.transport.sendto(data, addr)
-
     async def request(self, request, addr, integrity_key=None, retransmissions=None):
         """
         Execute a STUN transaction and return the response.
@@ -195,6 +188,10 @@ class StunProtocol(asyncio.DatagramProtocol):
         finally:
             del self.transactions[request.transaction_id]
 
+    async def send_data(self, data, addr):
+        self.__log_debug('> %s DATA %d', addr, len(data))
+        self.transport.sendto(data, addr)
+
     def send_stun(self, message, addr):
         """
         Send a STUN message.
@@ -224,7 +221,11 @@ class Connection:
     """
     def __init__(self, ice_controlling, components=1,
                  stun_server=None,
-                 turn_server=None, turn_username=None, turn_password=None, turn_transport='udp',
+                 turn_server=None,
+                 turn_username=None,
+                 turn_password=None,
+                 turn_ssl=False,
+                 turn_transport='udp',
                  use_ipv4=True, use_ipv6=True):
         self.ice_controlling = ice_controlling
         #: Local username, automatically set to a random value.
@@ -235,10 +236,12 @@ class Connection:
         self.remote_username = None
         #: Remote password, which you need to set.
         self.remote_password = None
+
         self.stun_server = stun_server
         self.turn_server = turn_server
         self.turn_username = turn_username
         self.turn_password = turn_password
+        self.turn_ssl = turn_ssl
         self.turn_transport = turn_transport
 
         # private
@@ -253,10 +256,10 @@ class Connection:
         self._local_candidates_start = False
         self._nominated = {}
         self._protocols = []
-        self._received = []
         self._remote_candidates = []
         self._remote_candidates_end = False
         self._query_consent_handle = None
+        self._queue = asyncio.Queue()
         self._tie_breaker = secrets.randbits(64)
         self._use_ipv4 = use_ipv4
         self._use_ipv6 = use_ipv6
@@ -432,18 +435,10 @@ class Connection:
         if not len(self._nominated):
             raise ConnectionError('Cannot receive data, not connected')
 
-        if not self._received:
-            fs = [protocol.recv_data() for protocol in self._protocols]
-            done, pending = await asyncio.wait(fs, return_when=asyncio.FIRST_COMPLETED)
-            for task in pending:
-                task.cancel()
-            for task in done:
-                result = task.result()
-                if result[0] is None:
-                    raise ConnectionError('Connection lost while receiving data')
-                self._received.append(result)
-
-        return self._received.pop(0)
+        result = await self._queue.get()
+        if result[0] is None:
+            raise ConnectionError('Connection lost while receiving data')
+        return result
 
     async def send(self, data):
         """
@@ -717,19 +712,23 @@ class Connection:
                 server_addr=self.turn_server,
                 username=self.turn_username,
                 password=self.turn_password,
+                ssl=self.turn_ssl,
                 transport=self.turn_transport)
             self._protocols.append(protocol)
 
             # add relayed candidate
             candidate_address = protocol.transport.get_extra_info('sockname')
+            related_address = protocol.transport.get_extra_info('related_address')
             protocol.local_candidate = Candidate(
-                foundation=candidate_foundation('relay', self.turn_transport, candidate_address[0]),
+                foundation=candidate_foundation('relay', 'udp', candidate_address[0]),
                 component=component,
-                transport=self.turn_transport,
+                transport='udp',
                 priority=candidate_priority(component, 'relay'),
                 host=candidate_address[0],
                 port=candidate_address[1],
-                type='relay')
+                type='relay',
+                related_address=related_address[0],
+                related_port=related_address[1])
             candidates.append(protocol.local_candidate)
 
         return candidates
@@ -770,6 +769,9 @@ class Connection:
                     self._query_consent_handle = None
                     return await self.close()
 
+    def data_received(self, data, component):
+        asyncio.ensure_future(self._queue.put((data, component)))
+
     def request_received(self, message, addr, protocol, raw_data):
         if message.message_method != stun.Method.BINDING:
             self.respond_error(message, addr, protocol, (400, 'Bad Request'))
